# yacon.views.right_control.py
# blame ctrudeau chr(64) arsensa.com
#
# Right pane in Nexus control panel.  Shows the contents of items selected in
# the site tree in the left panel.

import logging

from django.http import HttpResponse
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404

from yacon.decorators import superuser_required
from yacon.models.hierarchy import (Node, NodeTranslation, Menu,
    MenuItem, MenuItemTranslation)
from yacon.models.site import Site
from yacon.models.pages import MetaPage
from yacon.utils import JSONResponse

logger = logging.getLogger(__name__)

# ============================================================================
# Control Panel: Site Info Button Methods
# ============================================================================

@superuser_required
def site_info(request, site_id):
    site = get_object_or_404(Site, id=site_id)

    data = {
        'site':site,
        'alternate_languages':site.alternate_language.all(),
    }
    return render_to_response('yacon/nexus/ajax/site_info.html', data,
        context_instance=RequestContext(request))


# ============================================================================
# Control Panel: Node Selected Methods
# ============================================================================

@superuser_required
def node_info(request, node_id):
    node = Node.objects.get(id=node_id)
    if node == None:
        return HttpResponse('')

    default_path = node.node_to_path(node.site.default_language)
    if default_path == None:
        default_path = '<i>empty translation (%s)</i>' % \
            node.site.default_language.code
    data = {
        'node':node,
        'num_metapages':len(MetaPage.objects.filter(node=node)),
        'num_children':node.get_children_count(),
        'path_items':[],
        'default_path':default_path,
    }

    class PathItem(object):
        pass

    # find all of the aliases for this page
    langs = node.site.get_languages()
    for lang in langs:
        item = PathItem()
        item.lang = lang
        item.path = node.node_to_path(lang)
        item.translation = None
        if item.path:
            try:
                item.translation = NodeTranslation.objects.get(node=node, 
                    language=lang)
            except NodeTranslation.DoesNotExist:
                # no translations available, e.g. root node, do nothing
                pass

        if node.default_metapage != None:
            item.page = node.default_metapage.get_translation(lang)

        data['path_items'].append(item)

    return render_to_response('yacon/nexus/ajax/node_info.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def missing_node_translations(request, node_id):
    """Returns a JSON object hash of languages for which there are no
    translations in this Node."""
    node = get_object_or_404(Node, id=node_id)
    data = {}
    langs = [node.site.default_language, ]
    for lang in node.site.alternate_language.all():
        langs.append(lang)

    for translation in NodeTranslation.objects.filter(node=node):
        if translation.language in langs:
            langs.remove(translation.language)

    for lang in langs:
        data[lang.identifier] = lang.name

    return JSONResponse(data)


# ============================================================================
# Control Panel: MetaPage Selected Methods
# ============================================================================

@superuser_required
def metapage_info(request, metapage_id):
    metapage = get_object_or_404(MetaPage, id=metapage_id)

    default_page = metapage.get_default_translation()
    translated_pages = metapage.get_translations(ignore_default=True)

    try:
        menuitem = metapage.menuitem
    except MenuItem.DoesNotExist:
        menuitem = None

    data = {
        'title':'MetaPage Info',
        'metapage':metapage,
        'default_page':default_page,
        'translated_pages':translated_pages,
        'menuitem':menuitem,
    }

    return render_to_response('yacon/nexus/ajax/metapage_info.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def missing_metapage_translations(request, metapage_id):
    """Returns a JSON object hash of languages for which there are no
    translations in this MetaPage."""
    metapage = get_object_or_404(MetaPage, id=metapage_id)
    data = {}
    langs = [metapage.node.site.default_language, ]
    for lang in metapage.node.site.alternate_language.all():
        langs.append(lang)

    for page in metapage.get_translations():
        if page.language in langs:
            langs.remove(page.language)

    for lang in langs:
        data[lang.identifier] = lang.name

    return JSONResponse(data)


@superuser_required
def list_languages(request, site_id):
    """Returns a JSON object hash of all languages in the site."""
    site = get_object_or_404(Site, id=site_id)
    langs = [site.default_language, ]
    langs.extend(site.alternate_language.all())

    data = {}
    for lang in langs:
        data[lang.identifier] = lang.name

    return JSONResponse(data)


# ============================================================================
# Control Panel: Menu Selected Methods
# ============================================================================

@superuser_required
def menus_control(request):
    return render_to_response('yacon/nexus/ajax/menus_control.html', {}, 
        context_instance=RequestContext(request))


@superuser_required
def menu_info(request, menu_id):
    menu = get_object_or_404(Menu, id=menu_id)

    data = {
        'title':'Menu Info',
        'menu':menu,
    }

    return render_to_response('yacon/nexus/ajax/menu_info.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def toggle_menuitem_requires_login(request, menuitem_id):
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    menuitem.requires_login = not menuitem.requires_login
    menuitem.save()
    return HttpResponse()


@superuser_required
def move_menuitem_out(request, menuitem_id):
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    try:
        parent = menuitem.get_parent()
        menuitem.move(parent, 'right')
    except:
        logger.exception('problem moving item out %s', menuitem_id)

    return HttpResponse()


@superuser_required
def move_menuitem_up(request, menuitem_id):
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    try:
        sibling = menuitem.get_prev_sibling()
        menuitem.move(sibling, 'left')
    except:
        logger.exception('problem moving item up %s', menuitem_id)

    return HttpResponse()


@superuser_required
def move_menuitem_down(request, menuitem_id):
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    try:
        sibling = menuitem.get_next_sibling()
        menuitem.move(sibling, 'right')
    except:
        logger.exception('problem moving item down %s', menuitem_id)

    return HttpResponse()

# ============================================================================
# Control Panel: MenuItem Selected Methods
# ============================================================================

@superuser_required
def menuitem_info(request, menuitem_id):
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)

    default_translation = menuitem.get_default_translation()
    translated_items = menuitem.get_translations(ignore_default=True)

    data = {
        'title':'MenuItem Info',
        'menuitem':menuitem,
        'default_translation':default_translation,
        'translated_items':translated_items,
    }

    return render_to_response('yacon/nexus/ajax/item_info.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def missing_menuitem_translations(request, menuitem_id):
    """Returns a JSON object hash of languages for which there are no
    translations in this MenuItem."""
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    data = {}
    langs = [menuitem.menu.site.default_language, ]
    for lang in menuitem.menu.site.alternate_language.all():
        langs.append(lang)

    for translation in MenuItemTranslation.objects.filter(menuitem=menuitem):
        if translation.language in langs:
            langs.remove(translation.language)

    for lang in langs:
        data[lang.identifier] = lang.name

    return JSONResponse(data)



from django.conf.urls import patterns
from django.views.generic import TemplateView, RedirectView

from yacon import conf

urlpatterns = patterns('yacon.views.browser',
    (r'^ckeditor_browser/$', 'ckeditor_browser'),
    (r'^popup_browser/(.*)/$', 'popup_browser'),

    # left pane
    (r'^browser/tree_top/$', 'tree_top'),
    (r'^browser/sub_tree/$', 'sub_tree'),

    # right pane
    (r'^browser/root_control/(.*)/$', 'root_control'),
    (r'^browser/show_folder/$', 'show_folder'),
    (r'^browser/add_folder/(.*)/$', 'add_folder'),
    (r'^browser/remove_folder_warn/$', 'remove_folder_warn'),
    (r'^browser/remove_folder/$', 'remove_folder'),
    (r'^browser/remove_file/$', 'remove_file'),
    (r'^browser/image_edit/$', 'image_edit'),
    (r'^browser/image_edit_save/$', 'image_edit_save'),

    # upload 
    (r'^browser/upload_file/$', 'upload_file'),
    (r'^browser/user_upload_file/$', 'user_upload_file'),
)

urlpatterns += patterns('yacon.views.content',
    (r'^fetch_block/(\d+)/$', 'fetch_block'),
    (r'^fetch_owners/(\d+)/$', 'fetch_owners'),
    (r'^replace_block/$', 'replace_block'),
    (r'^replace_owner/$', 'replace_owner'),
    (r'^replace_title/$', 'replace_title'),
    (r'^remove_page/(\d+)/$', 'remove_page'),
    (r'^create_page/(\d+)/([^/]*)/([^/]*)/(.*)/$', 'create_page'),
    (r'^create_page_from_node/(\d+)/(\d+)/([^/]*)/([^/]*)/$', 
        'create_page_from_node'),
)

if conf.site.static_serve and \
        (conf.nexus.enabled or conf.site.examples_enabled):
    # enable static serving of pages for nexus and examples
    import os
    cur_dir = os.path.dirname(__file__)
    static_root = os.path.join(cur_dir, 'static')

    urlpatterns += patterns('',
        (r'^static/(?P<path>.*)$', 'django.views.static.serve',
            {'document_root':static_root}),
    )

if conf.nexus.enabled:
    # nexus tabs
    urlpatterns += patterns('yacon.views.nexus',
        (r'^$', RedirectView.as_view(url='/yacon/nexus/control_panel/')),
        (r'^nexus/$', RedirectView.as_view(url='/yacon/nexus/control_panel/')),

        # some of the JS is templated in order to be able to dynamically 
        #disable features
        (r'^nexus/site_control/$', TemplateView.as_view(
            template_name='yacon/nexus/templated_js/site_control.js',
            content_type='application/javascript')),
    )

    # -------------------
    # Control Panel
    urlpatterns += patterns('yacon.views.nexus',
        (r'^nexus/control_panel/$', 'control_panel'),
    )

    # control panel, left pane
    urlpatterns += patterns('yacon.views.left_control',
        (r'^nexus/control/get_sites/$', 'get_sites'),
        (r'^nexus/control/tree_top/(\d+)/$', 'tree_top'),
        (r'^nexus/control/tree_top_default_site/$', 'tree_top_default_site'),
        (r'^nexus/control/sub_tree/$', 'sub_tree'),
    )

    # control panel, right pane
    urlpatterns += patterns('yacon.views.right_control',
        (r'^nexus/control/site_info/(\d+)/$', 'site_info'),
        (r'^nexus/control/node_info/(\d+)/$', 'node_info'),
        (r'^nexus/control/metapage_info/(\d+)/$', 'metapage_info'),
        (r'^nexus/control/list_languages/(\d+)/$', 'list_languages'),
        (r'^nexus/control/menus_control/$', 'menus_control'),
        (r'^nexus/control/menu_info/(\d+)/$', 'menu_info'),
        (r'^nexus/control/menuitem_info/(\d+)/$', 'menuitem_info'),
        (r'^nexus/control/missing_node_translations/(\d+)/$', 
            'missing_node_translations'),
        (r'^nexus/control/missing_metapage_translations/(\d+)/$', 
            'missing_metapage_translations'),
        (r'^nexus/control/missing_menuitem_translations/(\d+)/$', 
            'missing_menuitem_translations'),
        (r'^nexus/control/move_menuitem_out/(\d+)/$', 'move_menuitem_out'),
        (r'^nexus/control/move_menuitem_up/(\d+)/$', 'move_menuitem_up'),
        (r'^nexus/control/move_menuitem_down/(\d+)/$', 'move_menuitem_down'),
        (r'^nexus/control/toggle_menuitem_requires_login/(\d+)/$',
            'toggle_menuitem_requires_login'),
    )

    # control panel, dialogs
    urlpatterns += patterns('yacon.views.dialogs',
        # node dialogs
        (r'^nexus/control/remove_folder_warn/(\d+)/$', 'remove_folder_warn'),
        (r'^nexus/control/remove_folder/(\d+)/$', 'remove_folder'),
        (r'^nexus/control/add_folder/(\d+)/(.*)/(.*)/$', 'add_folder'),
        (r'^nexus/control/add_page/(\d+)/(\d+)/(.*)/(.*)/$', 'add_page'),
        (r'^nexus/control/add_path/(\d+)/(.*)/(.*)/(.*)/$', 'add_path'),
        (r'^nexus/control/remove_path_warn/(\d+)/$', 'remove_path_warn'),
        (r'^nexus/control/remove_path/(\d+)/$', 'remove_path'),
        (r'^nexus/control/edit_path_warn/(\d+)/$', 'edit_path_warn'),
        (r'^nexus/control/edit_path/(\d+)/(.*)/(.*)/$', 'edit_path'),

        # metapage dialogs
        (r'^nexus/control/page_types/$', 'page_types'),
        (r'^nexus/control/remove_page_warn/(\d+)/$', 'remove_page_warn'),
        (r'^nexus/control/remove_page/(\d+)/$', 'remove_page'),
        (r'^nexus/control/add_translation/(\d+)/(.*)/(.*)/(.*)/$', 
            'add_translation'),
        (r'^nexus/control/make_default_metapage/(\d+)/$', 
            'make_default_metapage'),
        (r'^nexus/control/remove_page_translation/(\d+)/$', 
            'remove_page_translation'),
        (r'^nexus/control/menu_listing/(\d+)/$', 'menu_listing'),
        (r'^nexus/control/add_menuitem/(\d+)/(\d+)/(.*)/$', 'add_menuitem'),

        # site dialogs
        (r'^nexus/control/missing_site_languages/(\d+)/$', 
            'missing_site_languages'),
        (r'^nexus/control/site_languages/(\d+)/$', 'site_languages'),
        (r'^nexus/control/all_languages/$', 'all_languages'),
        (r'^nexus/control/add_site_lang/(\d+)/(.*)/$', 'add_site_lang'),
        (r'^nexus/control/edit_site/(\d+)/(.*)/(.*)/(.*)/$', 'edit_site'),
        (r'^nexus/control/add_site/(.*)/(.*)/(.*)/$', 'add_site'),

        # menu dialogs
        (r'^nexus/control/add_menu/(\d+)/(.*)/$', 'add_menu'),
        (r'^nexus/control/remove_menu_warn/(\d+)/$', 'remove_menu_warn'),
        (r'^nexus/control/remove_menu/(\d+)/$', 'remove_menu'),

        # menuitem dialogs
        (r'^nexus/control/remove_menuitem_translation/(\d+)/$', 
            'remove_menuitem_translation'),
        (r'^nexus/control/remove_menuitem_warn/(\d+)/$', 
            'remove_menuitem_warn'),
        (r'^nexus/control/remove_menuitem/(\d+)/$', 'remove_menuitem'),
        (r'^nexus/control/add_menuitem_translation/(\d+)/(.*)/(.*)/$', 
            'add_menuitem_translation'),
        (r'^nexus/control/rename_menuitem_translation/(\d+)/(.*)/$', 
            'rename_menuitem_translation'),
        (r'^nexus/control/create_menuitem_translation/(\d+)/(.*)/(.*)/$', 
            'create_menuitem_translation'),
    )

    # -------------------
    # Settings Panel
    urlpatterns += patterns('yacon.views.nexus',
        (r'^nexus/config_panel/$', 'config_panel'),
    )

    urlpatterns += patterns('yacon.views.config_panel',
        (r'^nexus/config/add_language/(.*)/(.*)/$', 'add_language'),
    )

    # -------------------
    # Users Panel
    urlpatterns += patterns('yacon.views.nexus',
        (r'^nexus/users_panel/$', RedirectView.as_view(
            url='/yacon/nexus/users/list_users/')),
    )

    urlpatterns += patterns('yacon.views.users_panel',
        (r'^nexus/users/list_users/$', 'list_users'),
        (r'^nexus/users/edit_user/(\d+)/$', 'edit_user'),
        (r'^nexus/users/add_user/$', 'add_user'),
        (r'^nexus/users/user_password/(\d+)/$', 'user_password'),
        (r'^nexus/users/su/(\d+)/$', 'switch_to_user'),
    )

    # -------------------
    # Uploads Panel
    urlpatterns += patterns('yacon.views.nexus',
        (r'^nexus/uploads_panel/$', 'uploads_panel'),
    )

if conf.site.examples_enabled:
    urlpatterns += patterns('',
        (r'^(examples/uploads/.+\.html)$', TemplateView.as_view()),
    )



# yacon.views.user.py
# blame ctrudeau chr(64) arsensa.com

import logging, urllib

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.forms.util import ErrorList
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils import formats

from yacon.decorators import post_required
from yacon.forms import CreatePageForm
from yacon.helpers import prepare_context, permission_to_edit_page
from yacon.models.hierarchy import BadSlug, Node
from yacon.models.pages import Block, Page, PageType, MetaPage
from yacon.utils import JSONResponse, get_profile

logger = logging.getLogger(__name__)

# ============================================================================
# Constants

PAGE_CONTEXT = None

# ============================================================================
# Generic Page Display Views
# ============================================================================

def display_page(request, uri=''):
    """Default page rendering method for the CMS.  Uses the request object to
    determine what site is being displayed and the uri passed in to find an
    page to render. """
    data = prepare_context(request, uri)
    page = data['site'].find_page(uri)
    
    if page == None:
        # no such page found for uri
        raise Http404('CMS did not contain a page for uri: %s' % uri)

    data.update({
        'page':page,
        'create_mode':False,
        'translations':page.other_translations(),
        'edit_permission':permission_to_edit_page(request, page, data),
    })

    return page.metapage.page_type.render(request, data)


def _create_page_from_node(request, data, node, page_type_id, language_code, 
        auto_slug):
    """Helper for creating pages, used by page create methods that use a node
    or a uri."""
    site = node.site
    auto_slug = bool(auto_slug)
    page_type = get_object_or_404(PageType, id=page_type_id)
    langs = site.get_languages(language_code)
    if len(langs) == 0:
        lang = site.default_language
    else:
        lang = langs[0]

    profile = get_profile(request.user)
    if not profile:
        raise Http404('user had no profile')

    if not profile.permission_to_create_page(page_type, node):
        raise Http404('permission to create was denied')

    if request.method == 'POST':
        form = CreatePageForm(request.POST)
        if form.is_valid():
            clean = form.cleaned_data
            title = clean.pop('title', '')
            slug = clean.pop('slug', '')
            auto_slug = clean.pop('auto_slug', '')
            if auto_slug:
                slug = title

            block_types = {}
            for block_type in page_type.block_types.all():
                block_types[block_type.key] = block_type

            block_hash = {}
            for key, value in clean.items():
                block_type = block_types.get(key, None)
                if not block_type:
                    raise Http404(('block type key "%s" ' % key +\
                        'not registered with PageType %s' % page_type))

                block_hash[block_type] = urllib.unquote_plus(value)

            # create the MetaPage and Page, then redirect to the page just 
            # created
            try:
                metapage = MetaPage.create_page(node, page_type, title, slug,
                    block_hash, owner=request.user, auto_slug=auto_slug)
                page = metapage.get_translation(lang)
                return HttpResponseRedirect(page.uri)
            except BadSlug:
                errors = form._errors.setdefault('slug', ErrorList())
                errors.append(('Slug did not validate, either malformed or '
                    'duplicate'))
    else: # GET
        form = CreatePageForm(initial={'auto_slug':auto_slug})

    data.update({
        'create_mode':True,
        'page_type':page_type,
        'create_form':form,
        'edit_permission':True,
        'page':None,
    })

    return page_type.render(request, data)


@login_required
def create_page(request, page_type_id, language_code, auto_slug, uri):
    data = prepare_context(request, uri)
    parsed_path = data['site'].parse_path(uri)
    node = parsed_path.node
    if not parsed_path.node:
        raise Http404('uri "%s" did not lead to valid node' % uri)

    return _create_page_from_node(request, data, node, page_type_id, 
        language_code, auto_slug)


@login_required
def create_page_from_node(request, node_id, page_type_id, language_code, 
        auto_slug):
    node = get_object_or_404(Node, id=node_id)
    langs = node.site.get_languages(language_code)
    if len(langs) == 0:
        lang = node.site.default_language
    else:
        lang = langs[0]
    uri = node.node_to_path(language=lang)
    data = prepare_context(request, uri)

    return _create_page_from_node(request, data, node, page_type_id, 
        language_code, auto_slug)

# ============================================================================
# Ajax Views
# ============================================================================

@login_required
def fetch_block(request, block_id):
    """Ajax view for getting block contents."""
    block = get_object_or_404(Block, id=block_id)
    if not request.user.is_superuser:
        # one of the pages associated with the block must belong to the user
        # logged in 
        pages = block.page_set.filter(owner=request.user)
        if len(pages) == 0:
            raise Http404('permission denied')

    return JSONResponse(block.content)


@login_required
@post_required
def replace_block(request):
    """Ajax view for submitting edits to a block."""
    if not request.REQUEST.has_key('block_id'):
        raise Http404('replace_block requires "block_id" parameter')

    if not request.REQUEST.has_key('content'):
        raise Http404('replace_block requires "content" parameter')

    try:
        # block parameter is 'block_X' where X is the id we're after
        block_id = request.POST['block_id'][6:]
        block = Block.objects.get(id=block_id)
    except Block.DoesNotExist:
        raise Http404('no block with id "%s"' % block_id)

    # check permissions
    if not request.user.is_superuser:
        # one of the pages associated with the block must belong to the user
        # logged in 
        pages = block.page_set.filter(owner=request.user)
        if len(pages) == 0:
            raise Http404('permission denied')

    # set the new block content
    block.content = urllib.unquote(request.POST['content'])
    block.save()

    last_updated_list = []
    for page in block.page_set.all():
        when = formats.date_format(page.last_updated, 'DATETIME_FORMAT')
        last_updated_list.append((page.id, when))

    result = {
        'success':True,
        'block_id':block.id,
        'last_updated_list':last_updated_list,
    }
    response = JSONResponse(result, extra_headers={'Cache-Control':'no-cache'})
    return response


@login_required
def fetch_owners(request, page_id):
    """Ajax view for getting users."""
    if not request.user.is_superuser:
        raise Http404('permission denied')

    page = get_object_or_404(Page, id=page_id)
    owners = []
    for user in User.objects.all():
        owners.append([user.id, user.username])

    result = {
        'selected':page.owner.id if page.owner else 0,
        'owners':owners,
    }
    return JSONResponse(result)


@login_required
@post_required
def replace_owner(request):
    """Ajax view for change a page's owner."""
    if not request.REQUEST.has_key('page_id'):
        raise Http404('replace_page requires "page_id" parameter')

    if not request.REQUEST.has_key('owner_id'):
        raise Http404('replace_page requires "owner_id" parameter')

    page = get_object_or_404(Page, id=request.POST['page_id'])

    # check permissions
    if not request.user.is_superuser:
        # only admin can change ownership
        raise Http404('permission denied')

    owner_id = request.POST['owner_id']
    if owner_id == '0':
        owner = None
    else:
        try:
            owner = User.objects.get(id=owner_id)
        except User.ObjectDoesNotExist:
            raise Http404('no such user id %s' % owner_id)

    page.owner = owner
    page.save()

    result = {
        'success':True,
        'last_updated':formats.date_format(page.last_updated, 
            'DATETIME_FORMAT'),
        'page_id':page.id,
    }
    return JSONResponse(result, extra_headers={'Cache-Control':'no-cache'})



@login_required
@post_required
def replace_title(request):
    """Ajax view for submitting edits to a block."""
    if not request.REQUEST.has_key('page_id'):
        raise Http404('replace_title requires "page_id" parameter')

    if not request.REQUEST.has_key('content'):
        raise Http404('replace_title requires "content" parameter')

    try:
        # page parameter is 'page_X' where X is the id we're after
        page_id = request.POST['page_id'][5:]
        page = Page.objects.get(id=page_id)
    except Page.DoesNotExist:
        raise Http404('no page with id "%s"' % page_id)

    # check permissions
    if not request.user.is_superuser:
        # page must belong to logged in user
        if page.owner != request.user:
            raise Http404('permission denied')

    # set the new block content
    page.title = urllib.unquote(request.POST['content'])
    page.save()

    result = {
        'success':True,
        'last_updated':formats.date_format(page.last_updated, 
            'DATETIME_FORMAT'),
        'page_id':page.id,
    }
    return JSONResponse(result, extra_headers={'Cache-Control':'no-cache'})


@login_required
def remove_page(request, page_id):
    page = get_object_or_404(Page, id=page_id)
    if not permission_to_edit_page(request, page):
        raise Http404('permission problem')

    page.delete()
    return HttpResponseRedirect('/')



# groupsq.models.py
#
# GroupSq (Group Squared) works with Django auth to provide groups of groups
# and classes of groups functionality.

from django.contrib.auth.models import User, Group
from django.db import models

from yacon.models.common import TimeTrackedModel

# ============================================================================

class GroupOfGroups_Base(TimeTrackedModel):
    """Base class for a group of groups that contains users, groups or groups 
    of groups."""

    # GofG can contain django users, django groups or another GofG
    users = models.ManyToManyField(User, blank=True)
    groups = models.ManyToManyField(Group, blank=True)
    group_of_groups = models.ManyToManyField(
        'yacon.GroupOfGroups', blank=True)

    class Meta:
        app_label = 'yacon'
        abstract = True

    # -----------------------------------------
    # Settr Methods
    def add(self, item):
        if isinstance(item, User):
            self.users.add(item)
            self.save()
        elif isinstance(item, Group):
            self.groups.add(item)
            self.save()
        elif isinstance(item, GroupOfGroups):
            self.group_of_groups.add(item)
            self.save()
        else:
            raise AttributeError(
                'parameter to add() was not a User, Group or GroupOfGroups')

    # -----------------------------------------
    # Gettr Methods

    def has_user(self, user):
        """Searches for users recursively within all relations for this GofG
        and returns true if the passed in user is found."""

        # check if the requested user is in our list
        if user in self.users.all():
            return True

        # check if the requested user is in one of our django groups
        for group in self.groups.all():
            if user in group.user_set.all():
                return True

        # go through our recursive relationships and try and find the user
        for gog in self.group_of_groups.all():
            if gog.has_user(user):
                return True

        return False

    def list_users(self):
        """Returns a list of users belonging to this GofG or any of its
        sub-relations"""
        users = []
        users.extend(list(self.users.all()))

        for group in self.groups.all():
            users.extend(list(group.user_set.all()))

        # recurse through any sub relations and find all users
        for gog in self.group_of_groups.all():
            users.extend(gog.list_users())

        return users


class GroupOfGroups(GroupOfGroups_Base):
    """Defines a group of groups that is unique by name"""

    name = models.CharField(max_length=80, unique=True)


class OwnedGroupOfGroups(GroupOfGroups_Base):
    """Defines a group of groups that is owned by a user.  The name parameter
    is no longer needed to be unique.  Used for things like "friends of
    George". """

    owner = models.ForeignKey(User, related_name='ownedgroupofgroups_owner_set')
    name = models.CharField(max_length=80)



# yacon.curators.py

from yacon.models.users import UserProfile
from yacon.forms import UpdateUserForm, AddUserForm

class UserCurator(object):
    profile_class = UserProfile
    update_form_class = UpdateUserForm
    add_form_class = AddUserForm



# yacon.definitions.py
#
# This file contains common values that aren't usually changed per
# installation and therefore shouldn't go in the django settings file

# max length of slugs
SLUG_LENGTH = 25

# max length of title
TITLE_LENGTH = 50

# bleach constants
ALLOWED_TAGS = [
    'a',
    'address',
    'b',
    'br',
    'blockquote',
    'code',
    'div',
    'em',
    'h1',
    'h2',
    'h3',
    'h4',
    'h5',
    'h6',
    'hr',
    'img',
    'i',
    'li',
    'ol',
    'p',
    'pre',
    'span',
    'strike',
    'strong',
    'sub',
    'sup',
    'table',
    'tbody',
    'thead',
    'tr',
    'td',
    'u',
    'ul',
]

ALLOWED_ATTRIBUTES = {
    'a':['href', 'title'],
    'img':['alt', 'src', 'width', 'height', 'style'],
    'table':['border', 'cellpadding', 'cellspacing', 'style', ],
    'td':['style', ],
    'tr':['style', ],
    'tbody':['style', ],
    'thead':['style', ],
    'p':['style', ],
    'span':['style', ],
    'div':['style', ],
}

ALLOWED_STYLES = [
    'background-color',
    'border-width',
    'border-height',
    'border-style',
    'color',
    'float',
    'font-family',
    'font-size',
    'height',
    'margin',
    'margin-bottom',
    'margin-left',
    'margin-right',
    'margin-top',
    'text-align',
    'width',
]



from django.conf import settings
from django.db import models
from django.utils.encoding import smart_unicode

import bleach


class SanitizedCharField(models.CharField):
    
    def __init__(self, allowed_tags=[], allowed_attributes=[],
                 allowed_styles=[], strip=False, 
                 *args, **kwargs):
        self._sanitizer_allowed_tags = allowed_tags
        self._sanitizer_allowed_attributes = allowed_attributes
        self._sanitizer_allowed_styles = allowed_styles
        self._sanitizer_strip = strip
        super(SanitizedCharField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        value = super(SanitizedCharField, self).to_python(value)
        value = bleach.clean(value, tags=self._sanitizer_allowed_tags,
            attributes=self._sanitizer_allowed_attributes, 
            styles=self._sanitizer_allowed_styles, strip=self._sanitizer_strip)
        return smart_unicode(value)


class SanitizedTextField(models.TextField):
    
    def __init__(self, allowed_tags=[], allowed_attributes=[], 
                 allowed_styles=[], strip=False, 
                 *args, **kwargs):
        self._sanitizer_allowed_tags = allowed_tags
        self._sanitizer_allowed_attributes = allowed_attributes
        self._sanitizer_allowed_styles = allowed_styles
        self._sanitizer_strip = strip
        super(SanitizedTextField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        value = super(SanitizedTextField, self).to_python(value)
        value = bleach.clean(value, tags=self._sanitizer_allowed_tags,
            attributes=self._sanitizer_allowed_attributes, 
            styles=self._sanitizer_allowed_styles, strip=self._sanitizer_strip)
        return smart_unicode(value)

    def get_prep_value(self, value):
        value = super(SanitizedTextField, self).get_prep_value(value)
        value = bleach.clean(value, tags=self._sanitizer_allowed_tags,
            attributes=self._sanitizer_allowed_attributes, 
            styles=self._sanitizer_allowed_styles, strip=self._sanitizer_strip)
        return value


if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^yacon\.sanitizer\.SanitizedCharField"])
    add_introspection_rules([], ["^yacon\.sanitizer\.SanitizedTextField"])



# yacon.models.content.py
import exceptions, logging, traceback

from django.http import Http404
from django.template import RequestContext, Template
from django.utils.safestring import mark_safe

logger = logging.getLogger(__name__)

# ============================================================================
# PermissionHandler Objects
# ============================================================================

class PermissionHandler(object):
    """Base class for permissions used by ContentHandler.  Default denies all
    permissions to everyone."""
    CAN_VIEW = 1;
    CAN_EDIT = 2;

    def has_permission(self, request, mode):
        logger.debug('returning False')
        return False

    def permission_denied_content(self, request):
        logger.debug('returning default denied message')
        return '<p>Permission denied to this component</p>'


class AlwaysYesPermissionHandler(PermissionHandler):
    """PermissionHandler that grants everyone permission always.  Probably
    shouldn't be used in production, good for testing."""
    def has_permission(self, request, mode): 
        logger.debug('returning True')
        return True


# ============================================================================
# ContentHandler Objects
# ============================================================================

class ContentRenderingException(exceptions.Exception):
    pass


class ContentHandler(object):
    """Base class for content handlers, these specify how to retrieve content
    and make it available to the page renderers."""

    def __init__(self, block_type, parms):
        self.parms = parms
        self.block_type = block_type

    def internal_render(self, request, context, block):
        """Method that inheritors should over-ride to return content.  Called
        by render()"""
        pass

    def render(self, request, context, block):
        """Wraps internal_render() by adding information to any exceptions 
        caught and marking what is returned as safe."""
        try:
            return mark_safe(self.internal_render(request, context, block))
        except Http404, h:
            # don't want special handling for 404s, just pass them up
            raise h
        except Exception, e:
            et = e.__class__.__name__
            ot = self.__class__.__name__
            msg = \
"""
An exception was caught while rendering using the user specified 
ContentHandler "%s".  The exception was: "%s" 
with the message:

%s

Typical causes are errors in the internal_render() method.
""" % (ot, et, e)

            cre = ContentRenderingException(msg)
            logger.exception(msg)
            raise cre


# ----------------------------------------------------------------------------
# System ContentHandlers

class FlatContent(ContentHandler):
    """ContentHandler for content that requires no permission checking or
    other work, essentially just outputs what is in the db for the block"""
    def internal_render(self, request, context, block):
        c = RequestContext(request)
        t = Template(block.content)
        return t.render(c)


class DynamicContent(ContentHandler):
    """ContentHandler for content generated by user defined function.  The
    name of the function and the module it is in should be passed in to the
    "parameters" block of the ContentHandler using the keys "module" and
    "function", respectively.  

    The function name will be called, expecting three parameters, the request,
    the context for the request and the block firing the ContentHandler.  The
    function should return a string containing the content to be rendered. """
    def internal_render(self, request, context, block):
        mod = __import__(self.parms['module'], 
            fromlist=[self.parms['function']])
        f = getattr(mod, self.parms['function'])
        try:
            return f(request, context, block)
        except Http404, h:
            # don't want special handling for 404s, just pass them up
            raise h
        except Exception, e:
            et = e.__class__.__name__
            msg = \
"""
An exception was caught during rendering using the DynamicContent content
handler.   The exception was: "%s" with the message:

%s

Stack trace was:

%s
""" % (et, e, traceback.format_exc())

            cre = ContentRenderingException(msg)
            raise cre



from math import log10

from django import template
from django.http import Http404
from django.core.paginator import Paginator, InvalidPage
from django.conf import settings

register = template.Library()

DEFAULT_PAGINATION = getattr(settings, 'PAGINATION_DEFAULT_PAGINATION', 20)
DEFAULT_WINDOW = getattr(settings, 'PAGINATION_DEFAULT_WINDOW', 4)
DEFAULT_ORPHANS = getattr(settings, 'PAGINATION_DEFAULT_ORPHANS', 0)
INVALID_PAGE_RAISES_404 = getattr(settings,
    'PAGINATION_INVALID_PAGE_RAISES_404', False)
MAX_CHARACTERS = getattr(settings, 'PAGINATION_MAX_CHARACTERS', 25)


def replace_context_value(context, key, value):
    # searches through all dictionaries in the context and replaces keys at
    # any level with the given value
    for d in context.dicts:
        if key in d:
            d[key] = value


def do_autopaginate(parser, token):
    """
    Splits the arguments to the autopaginate tag and formats them correctly.
    """
    split = token.split_contents()
    as_index = None
    context_var = None
    for i, bit in enumerate(split):
        if bit == 'as':
            as_index = i
            break
    if as_index is not None:
        try:
            context_var = split[as_index + 1]
        except IndexError:
            raise template.TemplateSyntaxError("Context variable assignment " +
                "must take the form of {%% %r object.example_set.all ... as " +
                "context_var_name %%}" % split[0])
        del split[as_index:as_index + 2]
    if len(split) == 2:
        return AutoPaginateNode(split[1])
    elif len(split) == 3:
        return AutoPaginateNode(split[1], paginate_by=split[2], 
            context_var=context_var)
    elif len(split) == 4:
        try:
            orphans = int(split[3])
        except ValueError:
            raise template.TemplateSyntaxError(u'Got %s, but expected integer.'
                % split[3])
        return AutoPaginateNode(split[1], paginate_by=split[2], orphans=orphans,
            context_var=context_var)
    else:
        raise template.TemplateSyntaxError('%r tag takes one required ' +
            'argument and one optional argument' % split[0])

class AutoPaginateNode(template.Node):
    """
    Emits the required objects to allow for Digg-style pagination.
    
    First, it looks in the current context for the variable specified, and using
    that object, it emits a simple ``Paginator`` and the current page object 
    into the context names ``paginator`` and ``page_obj``, respectively.
    
    It will then replace the variable specified with only the objects for the
    current page.
    
    .. note::
        
        It is recommended to use *{% paginate %}* after using the autopaginate
        tag.  If you choose not to use *{% paginate %}*, make sure to display the
        list of available pages, or else the application may seem to be buggy.
    """
    def __init__(self, queryset_var, paginate_by=DEFAULT_PAGINATION,
        orphans=DEFAULT_ORPHANS, context_var=None):
        self.queryset_var = template.Variable(queryset_var)
        if isinstance(paginate_by, int):
            self.paginate_by = paginate_by
        else:
            self.paginate_by = template.Variable(paginate_by)
        self.orphans = orphans
        self.context_var = context_var

    def render(self, context):
        key = self.queryset_var.var
        value = self.queryset_var.resolve(context)
        if isinstance(self.paginate_by, int):
            paginate_by = self.paginate_by
        else:
            paginate_by = self.paginate_by.resolve(context)
        paginator = Paginator(value, paginate_by, self.orphans)
        try:
            page_obj = paginator.page(context['request'].page)
        except InvalidPage:
            if INVALID_PAGE_RAISES_404:
                raise Http404('Invalid page requested.  If DEBUG were set to ' +
                    'False, an HTTP 404 page would have been shown instead.')
            context.dicts[0][key] = []
            context.dicts[0]['invalid_page'] = True
            return u''
        if self.context_var is not None:
            replace_context_value(context, self.context_var, 
                page_obj.object_list)
        else:
            replace_context_value(context, key, page_obj.object_list)

        # put the pagination objects into the first context dictionary so they
        # can be used across template blocks
        context.dicts[0]['paginator'] = paginator
        context.dicts[0]['page_obj'] = page_obj
        return u''


def _num_digits(value):
    return int(log10(value)) + 1


def paginate(context, window=DEFAULT_WINDOW, hashtag=''):
    """
    Renders the ``pagination/pagination.html`` template, resulting in a
    Digg-like display of the available pages, given the current page.  If there
    are too many pages to be displayed before and after the current page, then
    elipses will be used to indicate the undisplayed gap between page numbers.
    
    Requires one argument, ``context``, which should be a dictionary-like data
    structure and must contain the following keys:
    
    ``paginator``
        A ``Paginator`` or ``QuerySetPaginator`` object.
    
    ``page_obj``
        This should be the result of calling the page method on the 
        aforementioned ``Paginator`` or ``QuerySetPaginator`` object, given
        the current page.
    
    This same ``context`` dictionary-like data structure may also include:
    
    ``getvars``
        A dictionary of all of the **GET** parameters in the current request.
        This is useful to maintain certain types of state, even when requesting
        a different page.
        """
    try:
        paginator = context['paginator']
        page_obj = context['page_obj']
        page_range = paginator.page_range
        # Calculate the record range in the current page for display.
        records = {'first': 1 + (page_obj.number - 1) * paginator.per_page}
        records['last'] = records['first'] + paginator.per_page - 1
        if records['last'] + paginator.orphans >= paginator.count:
            records['last'] = paginator.count

        last_page = page_range[-1]
        digits_last = _num_digits(last_page)
        pages = [page_obj.number]
        difference = 1
        total_digits = _num_digits(page_obj.number)
        #import pudb; pudb.set_trace()
        while(True):
            # figure out how much extra we're adding due to elipses on the
            # left and right sides
            extra = 0
            if pages[0] != 1 and pages[0] != 2:
                extra = 4
            if pages[-1] != last_page and pages[-1] != last_page - 1:
                extra += 4

            # calculate space for dividers, there is a divider between each
            # item in the list but not at the ends
            dividers = len(pages) - 2
            if dividers < 0:
                dividers = 0

            extra += dividers

            if total_digits + extra + digits_last > MAX_CHARACTERS:
                # used up all our characters, add start and end parts and then
                # leave
                if pages[0] != 1:
                    # insert marker for elipses and the number one
                    if pages[0] != 2:
                        pages.insert(0, None)
                    pages.insert(0, 1)

                if pages[-1] != last_page:
                    # last number in our list isn't the last item, add elipses
                    # and the last item
                    if pages[-1] != last_page - 1:
                        pages.append(None)
                    pages.append(last_page)

                break

            # add something to the left and right (maybe)
            left = page_obj.number - difference
            if left > 0:
                pages.insert(0, left)
                total_digits += _num_digits(left)

            right = page_obj.number + difference
            if right <= last_page:
                pages.append(right)
                total_digits += _num_digits(right)

            difference += 1

            if len(pages) == last_page:
                break

        class PageHolder(object):
            pass

        before = None
        holders = []
        for page in pages:
            holder = PageHolder()
            holder.page = page
            holder.before_elipses = False
            holders.append(holder)

            if not page and before:
                before.before_elipses = True

            before = holder

        to_return = {
            'MEDIA_URL': settings.MEDIA_URL,
            'pages': pages,
            'pageholders':holders,
            'records': records,
            'page_obj': page_obj,
            'paginator': paginator,
            'hashtag': hashtag,
            'is_paginated': paginator.count > paginator.per_page,
        }
        if 'request' in context:
            getvars = context['request'].GET.copy()
            if 'page' in getvars:
                del getvars['page']
            if len(getvars.keys()) > 0:
                to_return['getvars'] = "&%s" % getvars.urlencode()
            else:
                to_return['getvars'] = ''
        return to_return
    except KeyError:
        return {}
    except AttributeError:
        return {}

register.inclusion_tag('yacon/pagination/pagination.html', takes_context=True)(
    paginate)
register.tag('autopaginate', do_autopaginate)



# yacon.views.nexus.py
#
# Nexus is the area for administrators to control the contents of the site,
# permissions, user management etc.  Not named the obvious "admin" to avoid
# conflicts with Django's admin features
#

import logging

from django.template import RequestContext
from django.shortcuts import render_to_response

from yacon.decorators import superuser_required
from yacon.models.common import Language

logger = logging.getLogger(__name__)

# ============================================================================
# Tab Views
# ============================================================================

@superuser_required
def control_panel(request):
    data = {
        'title':'Control Panel',
    }

    return render_to_response('yacon/nexus/control_panel.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def config_panel(request):
    langs = Language.objects.all().order_by('identifier')
    data = {
        'title':'Settings',
        'langs':langs,
    }

    return render_to_response('yacon/nexus/config_panel.html', data, 
        context_instance=RequestContext(request))


# users_panel redirects straight to list_users


@superuser_required
def uploads_panel(request):
    data = {
        'title':'Uploads',
        'base_template':'yacon/nexus_base.html',
        'choose_mode':'view',
        'popup':False,
    }
    request.session['choose_mode'] = 'view'
    request.session['image_only'] = False
    request.session['popup'] = False

    return render_to_response('yacon/browser/browser.html', data, 
        context_instance=RequestContext(request))



# yacon.tests.py

from django.utils import unittest
from django.core import management
from django.contrib.auth.models import User, Group

from yacon.models.common import Language
from yacon.models.site import Site, ParsedPath
from yacon.models.pages import Page, DoubleAliasException
from yacon.models.hierarchy import Node, BadSlug
from yacon.models.groupsq import GroupOfGroups, OwnedGroupOfGroups

# ============================================================================
# Yacon Management Commands Test Cases
# ============================================================================

class ManagementCommands(unittest.TestCase):
    def setUp(self):
        management.call_command('yacon_add_site', 'blah', 'blah')
        management.call_command('yacon_create_defaults')
        management.call_command('yacon_create_test_data')

    def test_check_commands(self):
        # check that the 'blah' site was created
        site = Site.objects.get(name='blah')
        self.assertTrue(site)

        # check that the default site was created
        site = Site.objects.get(name='Localhost Site')
        self.assertTrue(site)

        # check that test data was created
        pp = site.parse_path('/articles/health/')
        self.assertTrue(pp.node)

# ============================================================================
# Site and Hierachy Test Cases
# ============================================================================

class SiteTestCase(unittest.TestCase):
    def test_hierarchy(self):
        british = Language.factory(name='GB English', identifier='en-gb')
        french = Language.factory(name='French', identifier='fr')

        # create a test site
        site = Site.create_site('Test Site', 'foo', [british, french])
        self.assertTrue(site)

        # test languages were created properly
        lang = site.default_language
        self.assertEquals(lang, british)
        langs = site.get_languages()
        self.assertEquals(len(langs), 2)
        self.assertTrue(british in langs)
        self.assertTrue(french in langs)
        langs = site.get_languages('en')
        self.assertEquals(len(langs), 1)
        self.assertTrue(british in langs)

        # test adding and retrieving config
        site.add_config('foo', 'bar')
        values = site.get_config('foo')
        self.assertEquals(values[0], 'bar')
        self.assertEquals(len(values), 1)

        # add some child nodes
        child1 = site.doc_root.create_child('Child1', 'child1', {\
            french:('Enfant1', 'enfant1')})
        self.assertTrue(child1)
        child2 = site.doc_root.create_child('Child2', 'child2', {\
            french:('Enfant2', 'enfant2')})
        self.assertTrue(child2)
        grandchild1 = child1.create_child('Grandchild1', 'grandchild1', {\
            french:('Grandenfant1', 'grandenfant1')})
        self.assertTrue(grandchild1)
        grandchild2 = child1.create_child('Grandchild2', 'grandchild2', {\
            french:('Grandenfant2', 'grandenfant2')})
        self.assertTrue(grandchild2)

        # attempt to add with a bad slug
        self.assertRaises(BadSlug, site.doc_root.create_child, name='Child1', 
            slug='foo bar')

        # search for some paths, testing leading and trailing slashes ignored
        # properly and that right things are returned
        pp = site.parse_path('child1')
        self.assertEquals(pp.node, child1)
        self.assertEquals(pp.language, british)

        pp = site.parse_path('/child1')
        self.assertEquals(pp.node, child1)
        self.assertEquals(pp.language, british)

        pp = site.parse_path('/child1/')
        self.assertEquals(pp.node, child1)
        self.assertEquals(pp.language, british)

        pp = site.parse_path('/child1/grandchild2')
        self.assertEquals(pp.node, grandchild2)
        self.assertEquals(pp.language, british)

        # search for some paths using something besides default lang
        pp = site.parse_path('/enfant1/')
        self.assertEquals(pp.node, child1)
        self.assertEquals(pp.language, french)

        pp = site.parse_path('/enfant1/grandenfant2')
        self.assertEquals(pp.node, grandchild2)
        self.assertEquals(pp.language, french)

        # test path parser with a mismatched path
        pp = site.parse_path('/foo')
        self.assertEquals(pp.path, '/foo')
        self.assertEquals(pp.slugs_in_path, [])
        self.assertEquals(pp.slugs_after_item, ['foo'])
        self.assertEquals(pp.node, None)
        self.assertEquals(pp.page, None)
        self.assertEquals(pp.language, None)
        self.assertEquals(pp.item_type, ParsedPath.UNKNOWN)

        # test path parser with a good path, including bits past the node
        parsed = site.parse_path('/child1/grandchild2/foo/b')
        self.assertEquals(parsed.path, '/child1/grandchild2/foo/b')
        self.assertEquals(parsed.slugs_in_path, ['child1', 'grandchild2'])
        self.assertEquals(parsed.slugs_after_item, ['foo', 'b'])
        self.assertEquals(parsed.node, grandchild2)

        # test tree printing
        test_string = \
"""root (/)
   Child1 (child1)
      Grandchild1 (grandchild1)
      Grandchild2 (grandchild2)
   Child2 (child2)"""

        string = site.doc_root.tree_to_string()
        self.assertEquals(test_string, string)

        # test getting the path from a node
        test_string = "/child1/"
        string = child1.node_to_path()
        self.assertEquals(test_string, string)
        test_string = "/child1/grandchild2/"
        string = grandchild2.node_to_path()
        self.assertEquals(test_string, string)


# ============================================================================
# Page Test Cases
# ============================================================================

class PageTestCase(unittest.TestCase):
    def setUp(self):
        # test data was loaded during the ManagementCommands test, use that
        # data to test page interactions, start by getting some of the nodes
        # and pages themselves
        self.site = Site.objects.get(name='Localhost Site')

        # get our languages from the site
        self.english = self.site.get_languages('en')[0]
        self.french = self.site.get_languages('fr')[0]

        pp = self.site.parse_path('/articles/health/')
        self.health = pp.node
        self.assertIsInstance(self.health, Node)

        pp = self.site.parse_path('/articles/health/steak')
        self.steak = pp.page
        self.assertIsInstance(self.steak, Page)
        self.lesteak = pp.page.get_translation(self.french)
        self.assertIsInstance(self.lesteak, Page)

        pp = self.site.parse_path('/articles/health/smoking')
        self.smoking = pp.page
        self.assertIsInstance(self.smoking, Page)

    def test_tree(self):
        # ---------------------------------
        # Test in english

        # test invalid URI
        page = self.site.find_page('/foo/bar')
        self.assertEquals(page, None)

        # test a valid URI without a page slug but with default page
        page = self.site.find_page('/articles/health/')
        self.assertEquals(page, self.steak)
        self.assertEquals(page.metapage_alias, None)
        self.assertEquals(page.language, self.english)

        # test a valid URI without a page slug and without default page
        page = self.site.find_page('/articles/fitness/')
        self.assertEquals(page, None)

        # ---------------------------------
        # Test Multi-lingual
        page = self.site.find_page('/lesarticles/sante/lesteak')
        self.assertEquals(page, self.lesteak)
        self.assertEquals(page.metapage_alias, None)
        self.assertEquals(page.language, self.french)

    def test_aliases(self):
        # create an alias of an alias of the steak article
        aliases = self.health.site.doc_root.create_child('Aliases', 'aliases')
        mp_steak = self.steak.metapage
        mp_steak2 = mp_steak.create_alias(aliases)

        # resolve steak2 and make sure it comes back to steak
        self.assertEquals(mp_steak, mp_steak2.resolve_alias())

        # check whether it can be found
        page = self.site.find_page('/aliases/steak')
        self.assertEquals(page, self.steak)
        self.assertEquals(page.metapage_alias, mp_steak2)

        # ensure double alias disallowed
        with self.assertRaises(DoubleAliasException):
            mp_steak2.create_alias(aliases)


# ============================================================================
# GroupOfGroups Test Cases
# ============================================================================

class GroupOfGroupsTestCase(unittest.TestCase):
    def _user(self, username, email, password):
        try:
            u = User.objects.get(username=username)
        except User.DoesNotExist:
            u = User.objects.create_user(username, email, password)

        return u

    def _group(self, name, users):
        try:
            g = Group.objects.get(name=name)
        except Group.DoesNotExist:
            g = Group(name=name)
            g.save()

            for user in users:
                user.groups.add(g)
                user.save()

        return g

    def setUp(self):
        # create some users and groups for testing
        self.homer = self._user('homer', 'homer@powerplant.com', 'password')
        self.lenny = self._user('lenny', 'lenny@powerplant.com', 'password')
        self.carl = self._user('carl', 'carl@powerplant.com', 'password')
        self.waylon = self._user('waylon', 'waylon@powerplant.com', 'password')
        self.montgomery = self._user('montgomery', 'montgomery@powerplant.com', 
            'password')
        self.hans = self._user('hans', 'hans@powerplant.com', 'password')
        self.fritz = self._user('fritz', 'fritz@powerplant.com', 'password')

        users = [self.homer, self.lenny, self.carl]
        self.powerplant_peons = self._group('powerplant_peons', users)

    def test_gog(self):
        # create a new gog and put a user in it
        powerplant_owners = GroupOfGroups(name='powerplant_owners')
        powerplant_owners.save()
        powerplant_owners.add(self.montgomery)

        # test the has_user functionality
        result = powerplant_owners.has_user(self.montgomery)
        self.assertTrue(result)

        # test the list user functionality
        users = powerplant_owners.list_users()
        self.assertEquals(len(users), 1)
        self.assertTrue(self.montgomery in users)

        # create a new gog and add it as a subrelation
        germans = GroupOfGroups(name='germans')
        germans.save()
        germans.add(self.hans)
        germans.add(self.fritz)

        powerplant_owners.add(germans)

        # test the has_user functionality for the nested group
        result = powerplant_owners.has_user(self.hans)
        self.assertTrue(result)

        # test the list user functionality
        users = powerplant_owners.list_users()
        self.assertEquals(len(users), 3)
        self.assertTrue(self.montgomery in users)
        self.assertTrue(self.hans in users)
        self.assertTrue(self.fritz in users)

        # create a new gog that has a user, a group and a nested gog
        powerplant_employees = GroupOfGroups(name='powerplant_employees')
        powerplant_employees.save()
        powerplant_employees.add(self.waylon)
        powerplant_employees.add(powerplant_owners)
        powerplant_employees.add(self.powerplant_peons)

        # test the has_user functionality for each type of item
        result = powerplant_employees.has_user(self.waylon)
        self.assertTrue(result)
        result = powerplant_employees.has_user(self.hans)
        self.assertTrue(result)
        result = powerplant_employees.has_user(self.homer)
        self.assertTrue(result)

        # test the list user functionality
        users = powerplant_employees.list_users()
        self.assertEquals(len(users), 7)
        self.assertTrue(self.montgomery in users)
        self.assertTrue(self.hans in users)
        self.assertTrue(self.fritz in users)
        self.assertTrue(self.waylon in users)
        self.assertTrue(self.homer in users)
        self.assertTrue(self.lenny in users)
        self.assertTrue(self.carl in users)

    def test_owned_group(self):
        homer_friends = OwnedGroupOfGroups(owner=self.homer, name='friends')
        homer_friends.save()
        homer_friends.add(self.lenny)
        homer_friends.add(self.carl)

        hans_friends = OwnedGroupOfGroups(owner=self.hans, name='friends')
        hans_friends.save()
        hans_friends.add(self.fritz)

        tests = OwnedGroupOfGroups.objects.filter(owner=self.homer, 
            name='friends')
        friends = tests[0].list_users()
        self.assertEquals(len(friends), 2)
        self.assertTrue(self.lenny in friends)
        self.assertTrue(self.carl in friends)



# yacon.models.pages.py
import exceptions, logging, json

from django.contrib.auth.models import User
from django.db import models, IntegrityError, transaction
from django.shortcuts import render_to_response
from django.template import RequestContext

from yacon.definitions import (SLUG_LENGTH, TITLE_LENGTH, ALLOWED_TAGS,
    ALLOWED_ATTRIBUTES, ALLOWED_STYLES)
from yacon.loaders import dynamic_load
from yacon.models.common import Language, TimeTrackedModel
from yacon.sanitizer import SanitizedTextField

logger = logging.getLogger(__name__)

# ============================================================================
# Page Management Classes
# ============================================================================

class PageType(TimeTrackedModel):
    """Defines how a page is constructed, tied to a template for rendering."""
    name = models.CharField(max_length=25, unique=True)
    template = models.CharField(max_length=50, blank=True)
    dynamic = models.CharField(max_length=100, blank=True)
    block_types = models.ManyToManyField('yacon.BlockType')

    class Meta:
        app_label = 'yacon'

    def render(self, request, data):
        # if there is something in template, do a static render
        if self.template:
            return render_to_response(self.template, data, 
                context_instance=RequestContext(request))

        # dynamic content render, call the function that is registered to
        # return a response
        fn = dynamic_load(self.dynamic.encode('ascii', 'ignore'))
        response = fn(request, data)
        return response


class BadContentHandler(exceptions.Exception):
    """Exception raised when an attempt is made to load or manipulate a
    ContentHandler and something goes wrong.  See:
    BlockType.get_content_handler()
    """
    pass


class BlockType(TimeTrackedModel):
    """Contains the name of a ContentHandler object which is used for managing
    content on a page.  A PageType references to multiple BlockType objects"""
    name = models.CharField(max_length=25, unique=True)
    key = models.CharField(max_length=25, unique=True)

    # '%s.%s' % (mod, content_handler) should produce the fully qualified name
    # of an object that inherits from ContentHandler (or at least duck-types)
    module_name = models.CharField(max_length=100)
    content_handler_name = models.CharField(max_length=50)
    content_handler_parms = models.TextField(null=True, blank=True)

    def __init__(self, *args, **kwargs):
        try:
            parms = kwargs['content_handler_parms']
        except KeyError:
            # no content_handler_parms passed in, store empty dict
            parms = {}

        serialized = json.JSONEncoder().encode(parms)
        kwargs['content_handler_parms'] = serialized

        super(BlockType, self).__init__(*args, **kwargs)
        self.content_handler_instance = None

    def __unicode__(self):
        return u'BlockType(name=%s, key=%s)' % (self.name, self.key)

    class Meta:
        app_label = 'yacon'

    def get_content_handler_parms(self):
        """De-serializes and returns a hash representing the parameters for
        the content handler"""
        parms = json.loads(self.content_handler_parms)
        return parms

    def set_content_handler_parms(self, parms):
        serialized = json.JSONEncoder.encode(parms)
        self.content_handler_parms = serialized
        self.save()

    def get_content_handler(self):
        if self.content_handler_instance != None:
            return self.content_handler_instance

        # instantiate a content handler
        try:
            mod = __import__(self.module_name, globals(), locals(),
                [self.content_handler_name])
        except Exception, e:
            logger.exception('importing mod caused exception')
            t = e.__class__.__name__
            msg = \
"""
An exception was caught during the import of the user specified
module "%s".  The exception was: "%s" 
with the message:

%s

Exceptions during import are usually caused by syntax errors or 
import errors in the module.
""" % (self.module_name, t, e)

            bch = BadContentHandler(msg) 
            raise bch

        try:
            klass = getattr(mod, self.content_handler_name)
            logger.debug('found class for content handler')
        except Exception, e:
            logger.exception(\
                'getting class object for content handler caused exception')
            t = e.__class__.__name__
            msg = \
"""
An exception was caught during the retrieval of the user specified
class "%s" from the module "%s".  The 
exception was: "%s" with the message:

%s
""" % (self.content_handler_name, self.module_name, t, e)

            bch = BadContentHandler(msg)
            raise bch

        try:
            parms = self.get_content_handler_parms()
            self.content_handler_instance = klass(self, parms)
        except Exception, e:
            logger.exception('instantiating content handler caused exception')
            t = e.__class__.__name__
            msg = \
"""
An exception was caught during the instantiation of the user specified
class "%s" from the module "%s".  The 
exception was: "%s" with the message:

%s

Instantiation errors are usually caused by problems in the constructor.
""" % (self.module_name, self.content_handler_name, t, e)

            bch = BadContentHandler(msg)
            raise bch

        return self.content_handler_instance


class Block(TimeTrackedModel):
    """Defines a block of content for the CMS"""
    block_type = models.ForeignKey(BlockType)

    parameters = models.TextField(null=True, blank=True)
    content = SanitizedTextField(allowed_attributes=ALLOWED_ATTRIBUTES,
        allowed_tags=ALLOWED_TAGS, allowed_styles=ALLOWED_STYLES)

    # management of owner, groups, privileges etc. should go here (?)

    def __init__(self, *args, **kwargs):
        super(Block, self).__init__(*args, **kwargs)
        self.is_editable = False

    def __unicode__(self):
        return u'Block(block_type=%s, content=%s)' % (self.block_type,
            self.content)

    class Meta:
        app_label = 'yacon'

    def render(self, request, context):
        """Returns a rendered version of the block via its ContentHandler"""

        handler = self.block_type.get_content_handler()
        return handler.render(request, context, self)

# ============================================================================
# Page & Supporting Classes
# ============================================================================

class Translation(object):
    """Temporary class used to help construct Page and PageTranslation
    objects.  Used in conjunction with Page factory methods for creating a
    page and its translated contents.
    """
    def __init__(self, language, title, slug, block_hash):
        """Constructor

        :param language: Language object representing what language the blocks
            contained with are writen in
        :param title: title of the translated page content
        :param slug: slug of the translated page content
        :param block_hash: a hash mapping BlockType objects to content
        """
        self.language = language
        self.title = title
        self.slug = slug
        self.block_hash = block_hash


class Page(TimeTrackedModel):
    language = models.ForeignKey(Language, related_name='+')
    slug = models.CharField(max_length=SLUG_LENGTH)
    title = models.CharField(max_length=TITLE_LENGTH, blank=True, null=True)
    owner = models.ForeignKey(User, null=True)

    metapage = models.ForeignKey('yacon.MetaPage')

    # content for the page is stored in a series of blocks
    blocks = models.ManyToManyField(Block)

    def __init__(self, *args, **kwargs):
        super(Page, self).__init__(*args, **kwargs)
        self.after_slugs = None
        self.metapage_alias = None

    class Meta:
        app_label = 'yacon'
        unique_together = ('slug', 'metapage')

    def __unicode__(self):
        return 'Page(id=%s, path=%s)' % (self.id, self.uri)

    def save(self, *args, **kwargs):
        # ensure slug is unique at all times
        suffix = ''
        i = 2
        while(True):
            try:
                save_id = transaction.savepoint()
                self.slug += suffix
                super(Page, self).save(*args, **kwargs)
                break
            except IntegrityError:
                transaction.savepoint_rollback(save_id)
                suffix = '-%d' % i
                i += 1

    # ---------------------------
    # Search
    @classmethod
    def find(cls, node, slugs):
        """Seraches for a translated page within the given Node with the
        corresponding slug.

        :param node: Node to search for the translated page 
        :param slugs: list of slugs, first of which is the key for the page,
            remainder are stored in the returned object in case they're needed
            as parameters
        
        :returns: Page object corresponding to slugs[0] or None
        """
        try:
            page = Page.objects.get(slug=slugs[0], metapage__node=node)
            page.after_slugs = slugs[1:]
            return page
        except Page.DoesNotExist:
            pass

        # exact match was not found, check for aliases in the node
        metapages = MetaPage.objects.exclude(node=node, alias=None)
        for metapage in metapages:
            resolved = metapage.resolve_alias()
            try:
                page = Page.objects.get(slug=slugs[0], metapage=resolved)
                page.metapage_alias = metapage
                page.after_slugs = slugs[1:]
                return page
            except Page.DoesNotExist:
                pass

        # if you get here than no exact matches an no aliased matches
        return None

    @classmethod
    def find_by_page_type(cls, page_type, language=None, owner=None):
        """Returns all of the pages for the given page_type.  If the language
        parameter is provided then the results are restricted to just that
        language.

        :param page_type: returns pages that have this PageType object
        :param language: [optional] language to restrict the search by
        :param owner: [optional] user to restrict search by

        :returns: list of Page objects
        """
        pages = []
        kwargs = {
            'metapage___page_type':page_type
        }
        if language:
            kwargs['language'] = language
        if owner:
            kwargs['owner'] = owner

        pages = Page.objects.filter(**kwargs)
        return pages

    def other_translations(self):
        """Returns a list of Page objects that represent other translations
        for this page.

        :returns: list of Page objects
        """
        pages = Page.objects.filter(metapage=self.metapage)
        return pages.exclude(id=self.id)

    def get_translation(self, language):
        """Returns the translated version of this Page in the given language,
        or None if there is no such translation.

        :param language: Language object specifying what translation to look
            for

        :returns: Page object in language or None
        """
        return self.metapage.get_translation(language)

    # -------------------------------------------
    # Block Handling Methods

    def get_blocks(self, key):
        """Returns a list of blocks from this translated page

        :param key: key for blocks to be retrieved

        :returns: list of Block objects
        """
        return self.blocks.filter(block_type__key=key)

    def get_block_keys(self):
        """Returns a list of the keys for which there is Block content for
        this page

        :returns: list of keys
        """
        bts = self.blocks.value_list('block_type', flat=True).distinct()
        return [bt.key for bt in bts]


    def create_block(self, block_type, content):
        """Creates and saves a new Block and adds it to this translated page

        :param block_type: BlockType for the Block being created
        :param content: content of Block

        :returns: the created, saved and added Block
        """
        block = Block(block_type=block_type, content=content)
        block.save()
        self.blocks.add(block)
        return block

    # ---------------------------
    # Utility Methods

    @property
    def uri(self):
        """Returns a valid URI that leads to this translated page.  Note that
        this may not be the URI that was used to find this content.  The URI
        is reconstructed using the language of this translation to get the
        slug path from the Node joined with the slug for this page.  

        Due to the fact that you can traverse the system through aliases, that
        you can parameterize a page with extra slugs and you can ask a page
        for its translation, there is no guarantee that the URI the user
        visited is the one returned by this method.

        If a MetaPage alias is registered with this object then the URI
        returned corresponds to that object rather than the absolute MetaPage.
        The MetaPage alias is set if the Page.find() method was used to find
        this page and the URI included aliases MetaPages.

        :returns: URI that can get you to this translated page
        """
        if self.metapage_alias != None:
            node_part = self.metapage_alias.node.node_to_path(self.language)
        else:
            node_part = self.metapage.node.node_to_path(self.language)

        if not node_part:
            return None

        return '//%s%s%s' % (self.metapage.node.site.domain, node_part, 
            self.slug)

    @property
    def lang_code(self):
        return self.language.identifier.upper()

    @property
    def all_blocks(self):
        """Shortcut property method for "blocks.all()" so it can be called
        inside of a template."""
        return self.blocks.all()

    @property
    def last_updated(self):
        """Returns the most recent time stamp of this Page and all of its
        associated blocks."""
        latest = self.updated
        for block in self.blocks.all():
            if block.updated > latest:
                latest = block.updated

        return latest


class DoubleAliasException(Exception):
    pass


class MetaPage(TimeTrackedModel):
    """This class represents a collection of translated pages in the CMS along
    with its placement in the document hierarchy.  All pages, even if they
    only have one translation, have a MetaPage.
    """
    node = models.ForeignKey('yacon.Node')
    _page_type = models.ForeignKey(PageType, blank=True, null=True)
    alias = models.ForeignKey('yacon.MetaPage', blank=True, null=True)
    is_node_default = models.BooleanField(default=False)

    class Meta:
        app_label = 'yacon'
        verbose_name = 'MetaPage'

    def __unicode__(self):
        result = 'MetaPage(id=%s, ' % self.id
        page = self.get_default_translation()
        if page == None:
            result += 'path=%s, EMPTY TRANSLATION' % (
                self.node.node_to_path())
        else: 
            result += 'path=%s' % page.uri

        if self.is_alias():
            result += ', ALIAS'

        result += ')'
        return result

    # -------------------------------------------
    # Getters -- need to use these so aliased values resolve properly

    @property
    def page_type(self):
        if self.is_alias():
            return self.resolve_alias()._page_type

        return self._page_type

    # -------------------------------------------
    # MetaPage Factories

    @classmethod
    def create_page(cls, node, page_type, title, slug, block_hash, owner=None,
            auto_slug=False):
        """Creates, saves and returns a MetaPage object with a corresponding 
        Page object in the Site default language.

        :param node: node in Site hierarchy where the page lives
        :param page_type: PageType for this Page
        :param title: title for the page
        :param slug: slug for the page
        :param block_hash: hash of content mapping block_type to content
        :param owner: [optional] owner of the page being created
        :param auto_slug: [optional, default=False] True will automatically
            fix a slug that does not match the uniqueness property.  False
            raises an exception

        :returns: MetaPage object
        """
        translation = Translation(language=node.site.default_language,
            title=title, slug=slug, block_hash=block_hash)
        return cls.create_translated_page(node, page_type, [translation],
            owner, auto_slug)

    @classmethod
    def create_translated_page(cls, node, page_type, translations, owner=None,
            auto_slug=False):
        """Creates, saves and returns a MetaPage object with multiple
        Page object translations.

        :param node: node in Site hierarchy where the page lives
        :param page_type: PageType for this Page
        :param translations: a list of Translation objects
        :param owner: [optional] owner of the pages being created
        :param auto_slug: [optional, default=False] True will automatically
            fix a slug that does not match the uniqueness property.  False
            raises an exception

        :returns: MetaPage object

        :raises: BadSlug, if any of the slugs are not valid
        """
        # create the MetaPage
        metapage = MetaPage(node=node, _page_type=page_type)
        metapage.save()

        for tx in translations:
            slug = node.validate_slug(tx.slug, auto_slug)
            page = Page.objects.create(metapage=metapage, title=tx.title, 
                slug=slug, language=tx.language, owner=owner)

            # add the content to the translation
            for key, value in tx.block_hash.items():
                block = Block.objects.create(block_type=key, content=value)
                page.blocks.add(block)

        return metapage

    def create_alias(self, node):
        """An alias is a pointer to a MetaPage somewhere else in the Node
        hierarchy.  This creates an alias of the current MetaPage at the given
        point in the Node hierarchy.

        .. warning::
            Aliases should only be created using this method.  Aliasing an
            alias is not allowed to avoid complications and circular
            references.  If you create an alias by hand and get the reference
            wrong bad things could happen.

        :param node: node in Site hierarchy where the alias should be created

        :returns: MetaPage object that is an alias of self
        :raises: DoubleAliasException if you attempt to alias an alias
        """
        if self.is_alias():
            raise DoubleAliasException()

        metapage = MetaPage(node=node, alias=self)
        metapage.save()
        return metapage

    # -------------------------------------------
    # Setters
    def make_default_for_node(self):
        # find any other nodes that are marked as default and unset them
        mps = MetaPage.objects.filter(node=self.node, is_node_default=True)
        for mp in mps:
            mp.is_node_default = False
            mp.save()

        self.is_node_default = True
        self.save()

    # -------------------------------------------
    # Getters
    @property
    def default_title(self):
        """Returns the title of the Page for this MetaPage in the default
        translation, or None if there is no such Page."""
        default_page = self.get_default_translation()
        if default_page:
            return default_page.title

        return None

    # -------------------------------------------
    # Alias Methods
    def is_alias(self):
        """Returns True if this MetaPage is an alias of another.

        :returns: True if self is an alias
        """
        return self.alias != None

    def resolve_alias(self):
        """Returns the MetaPage that self's alias points to.  

        .. warning::
            Aliases should only be created with :func:`MetaPage.create_alias`.  
            This method does not check for circular references or aliases of
            aliases.  If the alias was not created properly this method could
            loop forever.

        :returns: MetaPage that this page is aliased to, or None
        """
        if not self.is_alias():
            return None

        return self.alias

    @property
    def has_alias(self):
        aliases = MetaPage.objects.filter(alias=self)
        return len(aliases) != 0

    # -------------------------------------------
    # Search Methods

    def get_translation(self, language):
        """Returns a Page object for this MetaPage in the given language.

        :param language: Language for the corresponding translation

        :returns: Page object or None
        """
        mp = self
        if self.is_alias():
            mp = self.resolve_alias()

        try:
            page = Page.objects.get(metapage=mp, language=language)
            if mp != self:
                page.metapage_alias = self
        except Page.DoesNotExist:
            return None

        return page

    def get_translations(self, ignore_default=False):
        """Returns a list of Page objects representing the translated content
        for this MetaPage.

        :param ignore_default: if True the default language will not be
            returned in the list; defaults to False

        :returns: list of Pages
        """
        mp = self
        if self.is_alias():
            mp = self.resolve_alias()

        pages = Page.objects.filter(metapage=mp)
        if ignore_default:
            # remove the default language from the list
            pages = pages.exclude(language=mp.node.site.default_language)

        if mp != self:
            for page in pages:
                page.metapage_alias = self

        return pages

    def get_default_translation(self):
        """Returns the Page object for the site default language

        :returns: Page object or None
        """
        mp = self
        if self.is_alias():
            mp = self.resolve_alias()

        try:
            page = Page.objects.get(metapage=mp, 
                language=mp.node.site.default_language)
            if mp != self:
                page.metapage_alias = self
        except Page.DoesNotExist:
            return None

        return page

    @property
    def has_missing_translations(self):
        """Returns True if there are languages in the site that there are no
        pages for."""
        return self.page_set.count() != self.node.site.language_count()



# yacon.models.common.py
import logging
from django.db import models
from django.conf import settings

logger = logging.getLogger(__name__)

# ============================================================================

class TimeTrackedModel(models.Model):
    """Abstract model for create & update fields.  """

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True
        app_label = 'yacon'


class Language(TimeTrackedModel):
    """This model defines languages that are acceptable in the Yacon CMS.  A
    language is made up of a language identifier which itself can be a
    language code with an optional addition of a country code for variations.

    For more on language identifiers and codes, see:

    http://www.i18nguy.com/unicode/language-identifiers.html
    """
    name = models.CharField(max_length=25)
    identifier = models.CharField(max_length=25, unique=True)

    class Meta:
        app_label = 'yacon'

    def __unicode__(self):
        return '%s (%s)' % (self.name, self.identifier.upper())

    # --------------------------------------------
    # Factories/Constructors
    @classmethod
    def factory(cls, name, identifier):
        """Returns a Language object by either fetching one with a matching
        identifier or creating a new one if no match is found.

        @param name: human readable name of Language (usually in
            English/installation default language
        @param identifier: unique language identifier

        @returns Language: matching Language class
        """
        try:
            lang = Language.objects.get(identifier=identifier)
        except Language.DoesNotExist:
            lang = Language(name=name, identifier=identifier)
            lang.save()

        return lang

    # --------------------------------------------
    # Getters
    @classmethod
    def default_language(cls):
        """Retrieves the default language for the installation (note this is
        different from the default language for an individual Site.  This is
        retrieved from the django settings file, if not found then 'en' is
        returned

        @returns: string with default language identifier
        """
        default = settings.LANGUAGE_CODE

        lang = None
        if default:
            try:
                lang = cls.objects.get(identifier=default)
                return lang
            except cls.DoesNotExist:
                # the default language doesn't exist as a class, create it
                try:
                    lang_dict = dict(settings.LANGUAGES)
                    lang = cls(name=lang_dict[default], identifier=default)
                    lang.save()
                    return lang
                except KeyError:
                    # setting wasn't in language dictionary, do nothing and
                    # we'll build the default one
                    pass

        # else: no language in settings file, create english
        lang = cls(name='English', identifier='en')
        lang.save()

        return lang

    @property
    def code(self):
        return self.identifier.upper()



# yacon.loader.py
#
# Utility function for loading modules.  In its own file to help avoid
# circular dependencies

import logging

logger = logging.getLogger(__name__)

# ============================================================================

def _name_splitter(name):
    fn_name = None
    mod_name = None
    parts = name.split('.')
    fn_name = parts[-1]
    mod_name = '.'.join(parts[:-1])

    return (fn_name, mod_name)


def dynamic_load(name):
    """Equiavlent of a "from X import Y" statement using dot notation to
    separate the module and thing being imported.  Returns the thing being
    imported.  

    Example: dynamic_load('foo.bar.Thing') would load the module "foo.bar" and
    return Thing from it.

    :param name: dot notation for import statement

    :returns: item imported, usually a function or a class
    :raises: will cause exceptions the same way an import will when things
        aren't found properly
    """
    (fn_name, mod_name) = _name_splitter(name)
    mod = __import__(mod_name, globals(), locals(), [fn_name])

    return getattr(mod, fn_name)


def dynamic_safe_load(name, variable_name, default):
    """Wraps dynamic_load() in a try/except, logs any errors

    :param name: import string to pass to dynamic_load()
    :param variable_name: name of thing trying to be loaded (rather than its
        contents, used in logger message
    :param default: value to return if an exception happens

    :returns: imported item
    """
    try:
        logger.debug('about to import %s', variable_name)
        return dynamic_load(name)
    except Exception:
        (fn_name, mod_name) = _name_splitter(name)
        msg = ('importing %s caused exception, setting:'
            '"%s", module name:"%s", function name: "%s"' )
        logger.exception(msg, variable_name, name, mod_name, fn_name)

    return default



# yacon.views.dialogs.py
# blame ctrudeau chr(64) arsensa.com
#
# Dialog box ajax methods for the nexus control panel.
#

import logging, json, urllib
from collections import OrderedDict

from django.db import IntegrityError
from django.http import HttpResponse
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404

from yacon.decorators import superuser_required
from yacon.models.common import Language
from yacon.models.hierarchy import (Node, BadSlug, NodeTranslation, Menu,
    MenuItem, MenuItemTranslation)
from yacon.models.site import Site
from yacon.models.pages import MetaPage, Page, PageType

logger = logging.getLogger(__name__)

# ============================================================================
# Helpers
# ============================================================================

def _pages_subtree_as_list(node, language, depth=1):
    """Returns a string representation as <li> and <ul> tags of the node 
    passed in and its children."""
    space = depth * '    '
    output = []

    if node.name == None:
        output.append('%s<li><i>empty translation (%s)</i></li>' % (space,
            language.code))
    else:
        output.append('%s<li>%s at %s</li>' % (space, node.name, 
            node.node_to_path()))

    if node.has_children():
        output.append('%s<ul>' % space)
        for child in node.get_children():
            output.append( _pages_subtree_as_list(child, language, depth+1))

        output.append('%s</ul>' % space)

    # leaf node, check for pages
    metapages = MetaPage.objects.filter(node=node)
    if len(metapages) != 0:
        output.append('%s<ul>' % space)
        for metapage in metapages:
            for page in metapage.get_translations():
                output.append('    %s<li>"%s" (<i>%s</i>)</li>' % \
                    (space, page.title, page.language.code))

        output.append('%s</ul>' % space)

    return '\n'.join(output)


def _menu_subtree_as_list(item, depth=1):
    """Returns a string representation as <li> and <ul> tags of the menuitem 
    passed in and its children."""
    space = depth * '    '
    output = []

    default = item.get_default_translation()
    translations = item.get_translations(ignore_default=True)

    if default:
        name = '%s (%s)' % (default.name, default.language.code)
    else:
        name = '<i>empty translation (%s)</i>' % \
            item.menu.site.default_language.code

    tx_names = []
    for tx in translations:
        tx_names.append('%s (%s)' % (tx.name, tx.language.code))

    txs = ''
    if translations:
        txs = ' [%s]' % ', '.join(tx_names)

    output.append('%s<li>%s%s</li>' % (space, name, txs))

    if item.has_children():
        output.append('%s<ul>' % space)
        for child in item.get_children():
            output.append( _menu_subtree_as_list(child, depth+1))

        output.append('%s</ul>' % space)

    return '\n'.join(output)


def page_as_li(page):
    return '   <li>"%s" at %s (%s)</li>' % (page.title, page.uri, 
        page.language.code)


def reachable_aliases(metapages, language=None):
    """Returns a string containing an html <ul> list of any aliased pages
    reachable from the "metapages" passed in.
    """
    aliases = MetaPage.objects.filter(alias__in=metapages)
    alias_list = []
    if len(aliases) != 0:
        alias_list.append('<ul>')
        for metapage in aliases:
            for page in metapage.get_translations():
                if language == None:
                    # return all languages
                    alias_list.append(page_as_li(page))
                else:
                    # return only languages that match what was passed in
                    if page.language == language:
                        alias_list.append(page_as_li(page))

        alias_list.append('</ul>')
    return '\n'.join(alias_list)


def reachable_from_node(node, language=None, include_aliases=True):
    """Returns a tuple of strings containing html <ul> lists of the Nodes and
    pages that are children of "node" and any MetaPages associated with these
    items.  

    :params node: node to find reachables for
    :params language: if None, returns all items, if specified restricts list
        to just those with the given language, defaults to None
    :params include_aliases: False to skip calculation of aliases, returns
        None for second item in tuple

    :returns: (node_list, alias_list)
    """
    alias_list = None
    if include_aliases:
        # find all of the MetaPages that would be unreachable
        nodes = list(node.get_descendants())
        nodes.append(node)
        metapages = MetaPage.objects.filter(node__in=nodes)

        # find anything that aliases one of the targeted metapages
        alias_list = reachable_aliases(metapages, language)

    node_list = \
"""<ul>
%s
</ul>""" % _pages_subtree_as_list(node, node.site.default_language)

    return (node_list, alias_list)

# ============================================================================
# Site Dialog Methods
# ============================================================================

@superuser_required
def missing_site_languages(request, site_id):
    """Returns a JSON hash of languages in the system but not in the given
    site."""
    site = get_object_or_404(Site, id=site_id)
    data = {}

    all_langs = set(Language.objects.all())
    site_langs = set([site.default_language, ])
    site_langs.update(site.alternate_language.all().order_by('identifier'))
    for lang in all_langs.difference(site_langs):
        data[lang.identifier] = lang.name

    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def site_languages(request, site_id):
    """Returns a JSON hash of languages for the given site."""
    site = get_object_or_404(Site, id=site_id)
    data = OrderedDict()

    langs = [site.default_language, ]
    langs.extend(site.alternate_language.all().order_by('identifier'))
    for lang in langs:
        data[lang.identifier] = lang.name

    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def all_languages(request):
    data = OrderedDict()
    langs = Language.objects.all().order_by('identifier')
    for lang in langs:
        data[lang.identifier] = lang.name

    return HttpResponse(json.dumps(data), content_type='application/json')

# ============================================================================
# Node Toolbar Dialog Box Methods
# ============================================================================

@superuser_required
def remove_folder_warn(request, node_id):
    """Ajax call that returns a listing of the nodes and pages that would be
    effected if node with id "node_id" is deleted."""
    node = get_object_or_404(Node, id=node_id)

    # find all of the MetaPages that would be removed
    (nodes, aliases) = reachable_from_node(node)
    data = {
        'nodes':nodes,
        'aliases':aliases,
    }

    return render_to_response('yacon/nexus/ajax/remove_folder_warning.html', 
        data, context_instance=RequestContext(request))


@superuser_required
def remove_folder(request, node_id):
    """Deletes node with id "node_id" and all of its children.  Unlinks any
    aliases to removed items."""
    node = get_object_or_404(Node, id=node_id)
    node.delete()

    return HttpResponse()


@superuser_required
def add_folder(request, node_id, title, slug):
    """Adds a new node underneath the given one."""
    node = get_object_or_404(Node, id=node_id)
    title = urllib.unquote(title)
    slug = urllib.unquote(slug)
    data = {}
    try:
        child = node.create_child(title, slug)
        data['key'] = 'node:%s' % child.id,
    except BadSlug, e:
        data['error'] = e.message
        
    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def add_page(request, node_id, page_type_id, title, slug):
    """Adds a new page underneath the given node."""
    node = get_object_or_404(Node, id=node_id)
    page_type = get_object_or_404(PageType, id=page_type_id)
    title = urllib.unquote(title)
    slug = urllib.unquote(slug)

    data = {}
    try:
        MetaPage.create_page(node, page_type, title, slug, {})
    except BadSlug, e:
        data['error'] = e.message
        
    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def add_path(request, node_id, lang, name, slug):
    """Adds a translation path to the given Node."""
    node = get_object_or_404(Node, id=node_id)
    name = urllib.unquote(name)
    slug = urllib.unquote(slug)

    data = {}
    try:
        langs = node.site.get_languages(lang)
        if len(langs) == 0:
            raise ValueError('Bad language selected')

        NodeTranslation.objects.create(node=node, slug=slug, name=name, 
            language=langs[0])
    except BadSlug, e:
        data['error'] = e.message
    except ValueError, e:
        data['error'] = e.message
        
    return HttpResponse(json.dumps(data), content_type='application/json')

# ============================================================================
# MetaPage Toolbar Dialog Box Methods
# ============================================================================

@superuser_required
def remove_page_warn(request, metapage_id):
    """Ajax call that returns a listing of the translated pages that would be
    effected if metapage with id "metapage_id" is deleted."""
    metapage = get_object_or_404(MetaPage, id=metapage_id)

    # find all of the Pages that would be removed
    pages = Page.objects.filter(metapage=metapage)
    page_list = ['<ul>']
    for page in pages:
        page_list.append(page_as_li(page))
    page_list.append('</ul>')

    # find anything that aliases the targeted metapage
    alias_list = reachable_aliases([metapage, ])

    data = {
        'metapage':metapage,
        'pages':'\n'.join(page_list),
        'aliases':alias_list,
    }

    return render_to_response('yacon/nexus/ajax/remove_page_warning.html', data,
        context_instance=RequestContext(request))


@superuser_required
def remove_page(request, metapage_id):
    """Deletes metapage with id "metapage_id" and all of its pages.  Unlinks any
    aliases to removed items."""
    metapage = get_object_or_404(MetaPage, id=metapage_id)
    metapage.delete()

    return HttpResponse()


@superuser_required
def add_translation(request, metapage_id, lang, title, slug):
    """Adds a translation to the given MetaPage."""
    metapage = get_object_or_404(MetaPage, id=metapage_id)
    title = urllib.unquote(title)
    slug = urllib.unquote(slug)

    data = {}
    try:
        langs = metapage.node.site.get_languages(lang)
        if len(langs) == 0:
            raise ValueError('Bad language selected')

        Page.objects.create(metapage=metapage, title=title, slug=slug,
            language=langs[0])
    except BadSlug, e:
        data['error'] = e.message
    except ValueError, e:
        data['error'] = e.message
        
    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def page_types(request):
    data = {}
    for page_type in PageType.objects.all():
        data[page_type.id] = page_type.name

    return HttpResponse(json.dumps(data), content_type='application/json')

# ============================================================================
# MenuItem Toolbar Dialog Box Methods
# ============================================================================

@superuser_required
def add_menu(request, site_id, name):
    """Adds a new Menu."""
    site = get_object_or_404(Site, id=site_id)
    name = urllib.unquote(name)
    Menu.objects.create(name=name, site=site)
    return HttpResponse()


@superuser_required
def add_menuitem_translation(request, menuitem_id, lang, name):
    """Adds a translation to the given menu item."""
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    name = urllib.unquote(name)

    data = {}
    try:
        langs = menuitem.menu.site.get_languages(lang)
        if len(langs) == 0:
            raise ValueError('Bad language selected')

        MenuItemTranslation.objects.create(menuitem=menuitem, name=name,
            language=langs[0])
    except ValueError, e:
        data['error'] = e.message
        
    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def menu_listing(request, metapage_id):
    """Returns a list of the menus for the site."""
    metapage = get_object_or_404(MetaPage, id=metapage_id)
    data = {}

    for menu in Menu.objects.filter(site=metapage.node.site):
        data[menu.id] = menu.name

    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def add_menuitem(request, menu_id, metapage_id, name):
    """Adds a new menuitem to the given menu."""
    menu = get_object_or_404(Menu, id=menu_id)
    metapage = get_object_or_404(MetaPage, id=metapage_id)
    name = urllib.unquote(name)

    lang = menu.site.default_language
    menu.create_child(metapage=metapage, translations={lang:name})
        
    return HttpResponse()


@superuser_required
def rename_menuitem_translation(request, translation_id, name):
    """Renames the given translation item."""
    name = urllib.unquote(name)
    tx = get_object_or_404(MenuItemTranslation, id=translation_id)
    tx.name = name
    tx.save()

    return HttpResponse()


@superuser_required
def create_menuitem_translation(request, menuitem_id, lang, name):
    """Renames the given translation item."""
    name = urllib.unquote(name)
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)

    data = {}
    try:
        langs = menuitem.menu.site.get_languages(lang)
        if len(langs) == 0:
            raise ValueError('Bad language selected')

        MenuItemTranslation.objects.create(menuitem=menuitem, name=name,
            language=langs[0])
    except ValueError, e:
        data['error'] = e.message
        
    return HttpResponse(json.dumps(data), content_type='application/json')

# ============================================================================
# Inline Action Dialog Methods
# ============================================================================

@superuser_required
def remove_path_warn(request, translation_id):
    """Ajax call that returns a listing of the nodes and pages that would be
    effected if translation with id "translation_id" is changed."""
    translation = get_object_or_404(NodeTranslation, id=translation_id)

    (nodes, aliases) = reachable_from_node(translation.node,
        translation.language, include_aliases=False)

    data = {
        'path':translation.get_path(),
        'nodes':nodes,
    }

    return render_to_response('yacon/nexus/ajax/remove_path_warning.html', data,
        context_instance=RequestContext(request))


@superuser_required
def remove_path(request, translation_id):
    translation = get_object_or_404(NodeTranslation, id=translation_id)
    translation.delete()
    return HttpResponse()


@superuser_required
def edit_path_warn(request, translation_id):
    """Ajax call that returns a listing of the nodes and pages that would be
    effected if translation with id "translation_id" is changed."""
    translation = get_object_or_404(NodeTranslation, id=translation_id)

    (nodes, aliases) = reachable_from_node(translation.node,
        translation.language, include_aliases=False)

    data = {
        'path':translation.get_path(),
        'nodes':nodes,
    }

    return render_to_response('yacon/nexus/ajax/edit_path_warning.html', data,
        context_instance=RequestContext(request))


@superuser_required
def edit_path(request, translation_id, name, slug):
    translation = get_object_or_404(NodeTranslation, id=translation_id)
    slug = urllib.unquote(slug)
    name = urllib.unquote(name)

    translation.slug = slug
    translation.name = name
    translation.save()
    return HttpResponse()


@superuser_required
def make_default_metapage(request, metapage_id):
    """Sets the given metapage to be the default page for its parent node."""
    metapage = get_object_or_404(MetaPage, id=metapage_id)
    metapage.make_default_for_node()

    return HttpResponse()


@superuser_required
def remove_page_translation(request, page_id):
    """Deletes page with id "page_id"."""
    page = get_object_or_404(Page, id=page_id)
    page.delete()

    return HttpResponse()


@superuser_required
def remove_menuitem_translation(request, tx_id):
    """Deletes menu item translation with id "tx_id". """
    tx = get_object_or_404(MenuItemTranslation, id=tx_id)
    tx.delete()

    return HttpResponse()


@superuser_required
def remove_menu_warn(request, menu_id):
    """Ajax call that returns a listing of the menuitems that would be
    effected if menu with id "menu_id" is removed.  """
    menu = get_object_or_404(Menu, id=menu_id)

    items = []
    for menuitem in menu.first_level.all():
        items.append(_menu_subtree_as_list(menuitem, 0))

    items_string = '<ul>\n%s\n</ul>' % '\n'.join(items)

    data = {
        'menu':menu,
        'items': items_string,
    }

    return render_to_response('yacon/nexus/ajax/remove_menu_warning.html', data,
        context_instance=RequestContext(request))


@superuser_required
def remove_menuitem_warn(request, menuitem_id):
    """Ajax call that returns a listing of the menuitems that would be
    effected if menuitem with id "menuitem_id" is removed.  """
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)

    items = _menu_subtree_as_list(menuitem, 0)
    items_string = '<ul>\n%s\n</ul>' % items

    data = {
        'menuitem':menuitem,
        'items': items_string,
    }

    return render_to_response('yacon/nexus/ajax/remove_menuitem_warning.html', 
        data, context_instance=RequestContext(request))


@superuser_required
def remove_menuitem(request, menuitem_id):
    """Deletes menu item with id "menuitem_id". """
    menuitem = get_object_or_404(MenuItem, id=menuitem_id)
    menuitem.delete()

    return HttpResponse()


@superuser_required
def remove_menu(request, menu_id):
    """Deletes menu with id "menu_id". """
    menu = get_object_or_404(Menu, id=menu_id)
    menu.delete()

    return HttpResponse()

# ============================================================================
# Site Management Methods
# ============================================================================

@superuser_required
def add_site_lang(request, site_id, identifier):
    site = get_object_or_404(Site, id=site_id)
    identifier = urllib.unquote(identifier)

    try:
        language = Language.objects.get(identifier=identifier)
        site.alternate_language.add(language)
    except Language.DoesNotExist:
        # only an ajax call, this dies and nothing happens anyhow
        pass

    return HttpResponse()


@superuser_required
def add_site(request, name, domain, lang_identifier):
    name = urllib.unquote(name)
    domain = urllib.unquote(domain)
    lang_identifier = urllib.unquote(lang_identifier)
    data = {}

    try:
        language = Language.objects.get(identifier=lang_identifier)
        Site.create_site(name, domain, [language, ])
    except Language.DoesNotExist:
        # only an ajax call, this dies and nothing happens anyhow
        data['error'] = 'Problem occurred with language code'
    except IntegrityError, e:
        if 'Key (name)' in e.message:
            data['error'] = 'Site with name "%s" already exists' % name
        elif 'Key (domain)' in e.message:
            data['error'] = 'Site for domain "%s" already exists' % domain
        else:
            data['error'] = 'An error occurred.  The database reported %s' % \
                e.message

    return HttpResponse(json.dumps(data), content_type='application/json')


@superuser_required
def edit_site(request, site_id, name, domain, lang_identifier):
    site = get_object_or_404(Site, id=site_id)
    name = urllib.unquote(name)
    domain = urllib.unquote(domain)
    lang_identifier = urllib.unquote(lang_identifier)

    data = {}
    try:
        language = Language.objects.get(identifier=lang_identifier)
        site.set_default_language(language)

        site.name = name
        site.domain = domain
        site.save()
    except Language.DoesNotExist:
        # only an ajax call, this dies and nothing happens anyhow
        data['error'] = 'Problem occurred with language code'
    except IntegrityError, e:
        if 'Key (name)' in e.message:
            data['error'] = 'Site with name "%s" already exists' % name
        elif 'Key (domain)' in e.message:
            data['error'] = 'Site for domain "%s" already exists' % domain
        else:
            data['error'] = 'An error occurred.  The database reported %s' % \
                e.message

    return HttpResponse(json.dumps(data), content_type='application/json')



# yacon.conf.py
import logging

from django.conf import settings

from yacon.loaders import dynamic_load, dynamic_safe_load

logger = logging.getLogger(__name__)

# ============================================================================
# Default Configuration Settings

SITE = {
    'static_serve':False,
    'tests_enabled':False,
    'examples_enabled':False,
    'local_jquery':False,
    'custom_enabled':True,
    'private_upload':None,
    'private_upload_url':None,
    'auto_thumbnails':{},
    'image_extensions':['jpg', 'jpeg', 'png', 'gif'],
    'logout_url':'/accounts/logout/',
    'text':{},
}

NEXUS = {
    'enabled':True,
    'add_site_disabled':False,
}

CUSTOM = {
    # 'page_context' handled specially
    # 'user_curator' handled specially
    'extra_admin_nav':[],
}

TEXT = {
    'content_edit_button':'Edit Content',
    'content_save_button':'Save',
    'content_cancel_button':'Cancel',
    'title_edit_button':'Edit Title',
    'title_save_button':'Save',
    'title_cancel_button':'Cancel',
}

# ============================================================================

def _fetch_setting(group, key):
    if hasattr(settings, 'YACON'):
        try:
            return settings.YACON[group][key]
        except KeyError:
            # either group or key wasn't there, do nothing and proceed to the
            # defaults
            pass

    # find a default value, either YACON wasn't in settings or it didn't have
    # an overridding key
    try:
        if group == 'site':
            return SITE[key]
        elif group == 'nexus':
            return NEXUS[key]
        elif group == 'custom':
            return CUSTOM[key]
        else:
            logger.error('bad group "%s" requested from yacon.conf')

    except KeyError:
        # bad key, do nothing
        pass

    return None


def _site(key):
    return _fetch_setting('site', key)


def _nexus(key):
    return _fetch_setting('nexus', key)


_PAGE_CONTEXT = None
_USER_CURATOR = None

def _custom(key):
    global _PAGE_CONTEXT, _USER_CURATOR

    custom_enabled = _fetch_setting('site', 'custom_enabled')
    value = _fetch_setting('custom', key)

    if key == 'page_context':
        if value and custom_enabled:
            # settings has overridden the page context, load it dynamically if
            # it  isn't already
            if not _PAGE_CONTEXT:
                _PAGE_CONTEXT = dynamic_safe_load(value,
                    'yacon.custom.page_context', None)
        else:
            # default page context
            _PAGE_CONTEXT = lambda x, y, z: ''

        return _PAGE_CONTEXT
    elif key == 'user_curator':
        if value and custom_enabled:
            # settings has overridden the curator, singleton load it
            if not _USER_CURATOR:
                _USER_CURATOR = dynamic_load(value)
        else:
            # default curator, singleton load it
            if not _USER_CURATOR:
                _USER_CURATOR = dynamic_load('yacon.curators.UserCurator')

        return _USER_CURATOR
    # else:
    return value

# ============================================================================
# Conf 

class Conf(object):
    def __init__(self, fn):
        self.fn = fn

    def __getattr__(self, key):
        value = self.fn(key)
        return value

site = Conf(_site)
nexus = Conf(_nexus)
custom = Conf(_custom)



from django.core.management.base import BaseCommand

class Command(BaseCommand):
    def handle(self, *args, **options):
        if len(args) != 1:
            print 'yacon_run -- run a given file within the django shell env'
            print 'usage:'
            print '   manage.py yacon_run filename'
            quit()

        execfile(args[0])
